#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only

from funnylog import logger


class ApplicationStartError(Exception):

    def __init__(self, result):
        """
        应用程序未启动
        :param result: 结果
        """
        err = f"应用程序未启动,{result}"
        logger.error(err)
        Exception.__init__(self, err)


class ApplicationError(Exception):

    def __init__(self, msg):
        """
        应用程序错误
        :param msg: 结果
        """
        logger.error(msg)
        Exception.__init__(self, msg)


class ElementNotFound(Exception):

    def __init__(self, name):
        """
        未找到元素
        :param name: 命令
        """
        err = f"未找到“{name}”元素！"
        logger.error(err)
        Exception.__init__(self, err)


class TemplateElementNotFound(Exception):

    def __init__(self, *name):
        """
        通过模板资源未匹配到对应元素
        :param name: 命令
        """
        err = "通过图片资源, 未在屏幕上匹配到元素"
        template = [f"{i}.png" for i in name]
        Exception.__init__(self, err, *template)


class TemplateElementFound(Exception):

    def __init__(self, *name):
        """
        通过模板资源匹配到对应元素
        :param name: 命令
        """
        err = "通过图片资源, 在屏幕中匹配到了不应该出现的元素"
        template = [f"{i}.png" for i in name]
        Exception.__init__(self, err, *template)


class TemplatePictureNotExist(Exception):

    def __init__(self, name):
        """
        文件不存在
        :param name: 命令
        """
        err = f"图片资源：{name} 文件不存在!"
        logger.error(err)
        Exception.__init__(self, err)


class AssertOptionError(AssertionError):

    def __init__(self, e):
        """
        断言操作失败
        """
        err = f"断言操作失败，未知原因：{e}"
        logger.error(err)
        AssertionError.__init__(self, err)


class ParamError(Exception):

    def __init__(self, name, msg):
        """
        参数错误
        :param name: 命令
        """
        err = f"参数错误：{name}、\n{msg}"
        logger.error(err)
        Exception.__init__(self, err)


class NoIconOfThisSize(Exception):

    def __init__(self, size):
        """
        参数错误
        :param size:
        """
        err = f"参数错误：{size}"
        logger.error(err)
        Exception.__init__(self, err)


class NoSuchWindowPositionParameter(Exception):

    def __init__(self, size):
        """
        参数错误
        :param size:
        """
        err = f"参数错误：{size}, 没有此窗口位置参数"
        logger.error(err)
        Exception.__init__(self, err)


class GetWindowInformation(Exception):

    def __init__(self, msg):
        """
        获取窗口信息错误
        """
        logger.error(msg)
        Exception.__init__(self, msg)


class NoSetReferencePoint(Exception):

    def __init__(self, msg):
        err = f"没有设置参考点！| {msg}"
        logger.error(err)
        Exception.__init__(self, err)


class ShellExecutionFailed(Exception):

    def __init__(self, msg):
        err = f"shell执行失败！| {msg}"
        logger.error(err)
        Exception.__init__(self, err)


class ElementExpressionError(Exception):

    def __init__(self, msg):
        err = f"查找元素表达式错误 <{msg}>"
        logger.error(err)
        Exception.__init__(self, err)


class NoSuchSkipMethodFound(Exception):

    def __init__(self, msg):
        err = f"未找到判断是否跳过的自定义方法 <{msg}>"
        logger.error(err)
        Exception.__init__(self, err)


class OcrTextRecognitionError(Exception):

    def __init__(self, msg):
        err = f"Ocr文字识别失败 <{msg}>"
        logger.error(err)
        Exception.__init__(self, err)

class YouQuPluginInstalledError(Exception):

    def __init__(self, msg):
        err = f"YouQu插件未安装 <{msg}>, 请尝试安装：pip install {msg}"
        logger.error(err)
        Exception.__init__(self, err)


if __name__ == '__main__':
    raise ElementNotFound



from youqu3.exception import YouQuPluginInstalledError

try:
    import requests

    HAS_REQUESTS = True
except ImportError:
    HAS_REQUESTS = False

if HAS_REQUESTS is False:
    raise YouQuPluginInstalledError("requests")


import click

from youqu3 import version


@click.group()
@click.help_option("-h", "--help", help="查看帮助信息")
@click.version_option(version, "-v", "--version", prog_name="YouQu3", help="查看版本号")
def cli(): ...


@cli.command()
@click.help_option("-h", "--help", help="查看帮助信息")
@click.option("-f", "--filepath", default=None, type=click.STRING,
              help="指定用例文件或目录路径执行，")
@click.option("-k", "--keywords", default=None, type=click.STRING,
              help="指定用例关键词执行，支持 'and/or/not' 逻辑表达式")
@click.option("-t", "--tags", default=None, type=click.STRING,
              help="指定用例标签执行，支持 'and/or/not' 逻辑表达式")
@click.option("--setup-plan", is_flag=True, default=False, help="")
def run(
        filepath,
        keywords,
        tags,
        setup_plan,
):
    """本地执行"""
    args = {
        "filepath": filepath,
        "keywords": keywords,
        "tags": tags,
        "setup_plan": setup_plan,
    }
    from youqu3.driver.run import Run
    Run(**args).run()


@cli.command()
@click.help_option("-h", "--help", help="查看帮助信息")
@click.option("-c", "--clients", default=None, type=click.STRING,
              help="远程机器信息:user@ip:password，多个机器之间用 '/' 连接")
@click.option("-f", "--filepath", default=None, type=click.STRING,
              help="指定用例文件路径执行")
@click.option("-k", "--keywords", default=None, type=click.STRING,
              help="指定用例关键词执行，支持 'and/or/not' 逻辑表达式")
@click.option("-t", "--tags", default=None, type=click.STRING,
              help="指定用例标签执行，支持 'and/or/not' 逻辑表达式")
def remote(
        clients,
        filepath,
        keywords,
        tags,
):
    """远程控制执行"""
    args = {
        "clients": clients,
        "filepath": filepath,
        "keywords": keywords,
        "tags": tags,
    }
    from youqu3.driver.remote import Remote
    Remote(**args).run()


@cli.command()
def init():
    """创建用例工程"""
    from youqu3.driver.init import Init
    Init().init()


@cli.command()
@click.help_option("-h", "--help", help="查看帮助信息")
@click.option("--python", default="3", type=click.STRING, help="指定虚拟环境的Python版本")
def envx(python):
    """虚拟环境安装"""
    from youqu3.driver.envx import envx as env
    env(python_version=python)


if __name__ == '__main__':
    cli()



from youqu3.exception import YouQuPluginInstalledError

try:
    from youqu_dogtail import DogtailUtils as Dogtail

    HAS_DOGTAIL = True
except ImportError:
    HAS_DOGTAIL = False

if HAS_DOGTAIL is False:
    raise YouQuPluginInstalledError("youqu-dogtail")



import subprocess
import sys

from youqu3 import exception
from youqu3 import logger
from youqu3 import setting


class Cmd:

    @staticmethod
    def _run(command, _input=None, timeout=None, check=False, **kwargs):
        with subprocess.Popen(command, **kwargs) as process:
            try:
                stdout, stderr = process.communicate(_input, timeout=timeout)
            except:
                process.kill()
                raise
            retcode = process.poll()
            if check and retcode:
                raise subprocess.CalledProcessError(
                    retcode, process.args, output=stdout, stderr=stderr
                )
        return subprocess.CompletedProcess(process.args, retcode, stdout, stderr)

    @classmethod
    def _getstatusoutput(cls, command, timeout):
        kwargs = {
            "shell": True,
            "stderr": subprocess.STDOUT,
            "stdout": subprocess.PIPE,
            "timeout": timeout,
        }
        try:
            if sys.version_info >= (3, 7):
                kwargs["text"] = True
            result = cls._run(command, **kwargs)
            data = result.stdout
            if isinstance(data, bytes):
                data = data.decode("utf-8")
            exitcode = result.returncode
        except subprocess.CalledProcessError as ex:
            data = ex.output
            exitcode = ex.returncode
        except subprocess.TimeoutExpired as ex:
            data = ex.__str__()
            exitcode = -1
        if data[-1:] == "\n":
            data = data[:-1]
        return exitcode, data

    @classmethod
    def run(
            cls,
            command: str,
            interrupt: bool = False,
            timeout: int = 25,
            print_log: bool = True,
            command_log: bool = True,
            return_code: bool = False
    ):
        """
         执行shell命令
        :param command: shell 命令
        :param interrupt: 命令异常时是否中断
        :param timeout: 命令执行超时
        :param out_debug_flag: 命令返回信息输出日志
        :param command_log: 执行的命令字符串日志
        :return: 返回终端输出
        """
        exitcode, stdout = cls._getstatusoutput(command, timeout=timeout)
        if command_log:
            logger.debug(command)
        if exitcode != 0 and interrupt:
            raise exception.ShellExecutionFailed(stdout)
        if print_log and stdout:
            logger.debug(stdout)
        if return_code:
            return stdout, exitcode
        return stdout

    @staticmethod
    def expect_run(
            cmd: str,
            events: dict,
            return_code=False,
            timeout: int = 30
    ):
        """
        expect_run(
            "ssh username@machine_ip 'ls -l'",
            events={'password':'secret\n'}
        )
        如果 return_code=True，返回 (stdout, return_code)
        """
        import pexpect
        return pexpect.run(
            cmd,
            events=events,
            withexitstatus=return_code,
            timeout=timeout
        )

    @classmethod
    def sudo_run(
            cls,
            command,
            password: str = None,
            interrupt: bool = False,
            timeout: int = 25,
            print_log: bool = True,
            command_log: bool = True,
            return_code: bool = False
    ):
        if password is None:
            password = setting.PASSWORD
        return cls.run(
            f"echo '{password}' | sudo -S {command}",
            interrupt=interrupt,
            timeout=timeout,
            print_log=print_log,
            command_log=command_log,
            return_code=return_code
        )


class RemoteCmd:

    def __init__(self, user: str, ip: str, password: str, connect_timeout: int = None):
        self.user = user
        self.ip = ip
        self.password = password
        self.connect_timeout = connect_timeout

    def remote_run(self, cmd: str, return_code: bool = False):
        try:
            from fabric import Connection
        except ImportError:
            raise exception.YouQuPluginInstalledError("fabric")
        c = Connection(
            host=self.ip,
            user=self.user,
            connect_timeout=self.connect_timeout,
            connect_kwargs={'password': self.password},
        )
        res = c.run(cmd)
        if return_code:
            return res.stdout, res.return_code
        return res.stdout

    def remote_sudo_run(self, cmd: str, return_code: bool = False):
        try:
            from fabric import Connection, Config
        except ImportError:
            raise exception.YouQuPluginInstalledError("fabric")
        c = Connection(
            host=self.ip,
            user=self.user,
            config=Config(overrides={'sudo': {'password': self.password}}),
            connect_timeout=self.connect_timeout,
            connect_kwargs={'password': self.password},
        )
        res = c.sudo(cmd)
        if return_code:
            return res.stdout, res.return_code
        return res.stdout

    def remote_expect_run(self):
        # TODO
        ...



#!/usr/bin/env python3
# _*_ coding:utf-8 _*_
# SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
# SPDX-License-Identifier: GPL-2.0-only
import os
import pathlib
import re
import shutil
from time import strftime

from youqu3 import setting



class Init:

    @staticmethod
    def copy_template_to_apps():
        # if os.listdir("."):
        #     if input("当前目录存在且里面存在文件，请确认是否清空（Y/N）：") in ("y", "Y"):
        #         os.system(f"rm -rf ./*")
        #     else:
        #         exit(0)
        os.system(f"cp -r {setting.TPL_PATH}/* .")
        os.system(f"cp {setting.TPL_PATH}/.gitignore-tpl  .")

    @classmethod
    def init(cls):
        cls.copy_template_to_apps()
        dirname = pathlib.Path(".").parent.name
        for root, dirs, files in os.walk("."):
            for file in files:
                app_name = dirname.lower()

                if file.endswith("-tpl"):

                    if "${app_name}" in file:
                        new_file = re.sub(
                            r"\${app_name}", app_name, file
                        )
                        shutil.move(f"{root}/{file}", f"{root}/{new_file}")
                        file = new_file

                    if ".py-tpl" in file or ".csv-tpl" in file:
                        with open(f"{root}/{file}", "r") as f:
                            codes = f.readlines()
                        new_codes = []
                        for code in codes:
                            if "${APP_NAME}" in code:
                                code = re.sub(r"\${APP_NAME}", app_name.upper(), code)
                            if "${app_name}" in code:
                                code = re.sub(r"\${app_name}", app_name, code)
                            if "${AppName}" in code:
                                code = re.sub(r"\${AppName}", app_name.title().replace("_", "").replace("-", ""), code)
                            if "${USER}" in code:
                                code = re.sub(r"\${USER}", setting.USERNAME, code)
                            if "${DATE}" in code:
                                code = re.sub(r"\${DATE}", strftime("%Y/%m/%d"), code)
                            if "${TIME}" in code:
                                code = re.sub(r"\${TIME}", strftime("%H:%M:%S"), code)
                            if "${FIXEDCSVTITLE}" in code:
                                code = re.sub(
                                    r"\${FIXEDCSVTITLE}",
                                    ",".join([i.value for i in setting.FixedCsvTitle]),
                                    code,
                                )
                            new_codes.append(code)
                        with open(f"{root}/{file}", "w") as f:
                            f.writelines([i for i in new_codes])

                    shutil.move(f"{root}/{file}", f"{root}/{file}".rstrip("-tpl"))

        tree = Tree(
            f":open_file_folder: [link file://{dirname}]{dirname}",
            guide_style="bold bright_blue",
        )
        walk_directory(pathlib.Path(dirname), tree)
        print(tree)



from youqu3 import exception

class MouseKeyChainMixin:
    x = None
    y = None
    result = None

    @classmethod
    def _check_xy(cls):
        if cls.x is None and cls.y is None:
            raise exception.ElementNotFound("坐标未找到")

    @classmethod
    def click(cls):
        cls._check_xy()
        from youqu_mousekey import MouseKey
        MouseKey.click(cls.x, cls.y)
        return cls

    @classmethod
    def right_click(cls):
        cls._check_xy()
        from youqu_mousekey import MouseKey
        MouseKey.right_click(cls.x, cls.y)
        return cls

    @classmethod
    def double_click(cls):
        cls._check_xy()
        from youqu_mousekey import MouseKey
        MouseKey.double_click(cls.x, cls.y)
        return cls

    @classmethod
    def center(cls):
        if cls.x is None and cls.y is None:
            return None
        return cls.x, cls.y



import random

from youqu3.exception import YouQuPluginInstalledError

try:
    from pdocr_rpc import OCR as _OCR
    from pdocr_rpc import setting as ocr_setting

    from youqu3 import setting

    HAS_OCR = True
except ImportError:
    HAS_OCR = False

if HAS_OCR is False:
    raise YouQuPluginInstalledError("pdocr-rpc")

from youqu3.mkmixin import MouseKeyChainMixin


class OCR(MouseKeyChainMixin):

    ocr_setting.NETWORK_RETRY = int(setting.OCR_NETWORK_RETRY)
    ocr_setting.PAUSE = float(setting.OCR_PAUSE)
    ocr_setting.TIMEOUT = float(setting.OCR_TIMEOUT)
    ocr_setting.MAX_MATCH_NUMBER = int(setting.OCR_MAX_MATCH_NUMBER)
    ocr_setting.PORT = setting.OCR_PORT
    _ocr_servers = [i.strip() for i in setting.OCR_SERVER_HOST.split("/") if i]

    @classmethod
    def ocr(cls, *args, **kwargs):
        """ocr load balance"""
        servers = cls._ocr_servers
        while servers:
            ocr_setting.SERVER_IP = random.choice(servers)
            if _OCR.check_connected() is False:
                servers.remove(ocr_setting.SERVER_IP)
                ocr_setting.SERVER_IP = None
            else:
                break
        if ocr_setting.SERVER_IP is None:
            raise EnvironmentError(f"所有OCR服务器不可用: {cls._ocr_servers}")
        cls.result = _OCR.ocr(*args, **kwargs)

        if isinstance(cls.result, tuple):
            cls.x, cls.y = cls.result

        return cls


if __name__ == '__main__':
    OCR.ocr("所有").center()